@@ -37,12 +37,10 @@
 #include <sys/time.h>
 #include <sys/utsname.h>
 #include <time.h>
-#include <locale.h>
 
 /* Legal conversion specifiers, as specified in the C standard. */
 #define C_STANDARD_STRFTIME_CHARACTERS "aAbBcdHIjmMpSUwWxXyYZ"
 #define C_STANDARD_NUMERIC_STRFTIME_CHARACTERS "dHIjmMSUwWyY"
-#define SUS_EXTENDED_STRFTIME_MODIFIERS "EO"
 
 #define SAFE_SHELL_CHARACTERS "-_0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
 
@@ -54,6 +52,72 @@ typedef struct {
 
 static GList *hash_tables_to_free_at_exit;
 
+/* We will need this for eel_unsetenv if there is no unsetenv. */
+#if !defined (HAVE_SETENV)
+extern char **environ;
+#endif
+
+/**
+ * eel_setenv:
+ * 
+ * Adds "*name=*value" to the environment
+ *
+ * Returns: see "putenv" - 
+ * 
+ **/
+int
+eel_setenv (const char *name, const char *value, gboolean overwrite)
+{
+#if defined (HAVE_SETENV)
+	return setenv (name, value, overwrite);
+#else
+	char *string;
+	
+	if (! overwrite && g_getenv (name) != NULL) {
+		return 0;
+	}
+	
+	/* This results in a leak when you overwrite existing
+	 * settings. It would be fairly easy to fix this by keeping
+	 * our own parallel array or hash table.
+	 */
+	string = g_strconcat (name, "=", value, NULL);
+	return putenv (string);
+#endif
+}
+
+/**
+ * eel_unsetenv:
+ * 
+ * Removes *name from the environment
+ * 
+ **/
+void
+eel_unsetenv (const char *name)
+{
+#if defined (HAVE_SETENV)
+	unsetenv (name);
+#else
+	int i, len;
+
+	len = strlen (name);
+	
+	/* Mess directly with the environ array.
+	 * This seems to be the only portable way to do this.
+	 */
+	for (i = 0; environ[i] != NULL; i++) {
+		if (strncmp (environ[i], name, len) == 0
+		    && environ[i][len] == '=') {
+			break;
+		}
+	}
+	while (environ[i] != NULL) {
+		environ[i] = environ[i + 1];
+		i++;
+	}
+#endif
+}
+
 /**
  * eel_g_date_new_tm:
  * 
@@ -103,12 +167,10 @@ eel_strdup_strftime (const char *format, struct tm *time_pieces)
 {
 	GString *string;
 	const char *remainder, *percent;
-	char code[4], buffer[512];
+	char code[3], buffer[512];
 	char *piece, *result, *converted;
 	size_t string_length;
 	gboolean strip_leading_zeros, turn_leading_zeros_to_spaces;
-	char modifier;
-	int i;
 
 	/* Format could be translated, and contain UTF-8 chars,
 	 * so convert to locale encoding which strftime uses */
@@ -154,17 +216,6 @@ eel_strdup_strftime (const char *format, struct tm *time_pieces)
 			turn_leading_zeros_to_spaces = FALSE;
 			break;
 		}
-
-		modifier = 0;
-		if (strchr (SUS_EXTENDED_STRFTIME_MODIFIERS, *remainder) != NULL) {
-			modifier = *remainder;
-			remainder++;
-
-			if (*remainder == 0) {
-				g_warning ("Unfinished %%%c modifier passed to eel_strdup_strftime", modifier);
-				break;
-			}
-		} 
 		
 		if (strchr (C_STANDARD_STRFTIME_CHARACTERS, *remainder) == NULL) {
 			g_warning ("eel_strdup_strftime does not support "
@@ -177,15 +228,9 @@ eel_strdup_strftime (const char *format, struct tm *time_pieces)
 		 * of 512 bytes, which is probably OK. There's no
 		 * limit on the total size of the result string.
 		 */
-		i = 0;
-		code[i++] = '%';
-		if (modifier != 0) {
-#ifdef HAVE_STRFTIME_EXTENSION
-			code[i++] = modifier;
-#endif
-		}
-		code[i++] = *remainder;
-		code[i++] = '\0';
+		code[0] = '%';
+		code[1] = *remainder;
+		code[2] = '\0';
 		string_length = strftime (buffer, sizeof (buffer),
 					  code, time_pieces);
 		if (string_length == 0) {
@@ -339,20 +384,6 @@ eel_g_str_list_alphabetize (GList *list)
 	return g_list_sort (list, (GCompareFunc) eel_strcoll);
 }
 
-int
-eel_g_str_list_index (GList *str_list,
-		      const char *str)
-{
-	int i;
-	GList *l;
-	for (i = 0, l = str_list; l != NULL; l = l->next, i++) {
-		if (!strcmp (str, (const char*)l->data)) {
-			return i;
-		}
-	}
-	return -1;
-}
-
 /**
  * eel_g_list_free_deep_custom
  *
@@ -918,103 +949,6 @@ eel_remove_weak_pointer (gpointer pointer_location)
 	*object_location = NULL;
 }
 
-/* Get the filename encoding, returns TRUE if utf8 */
-
-typedef struct _EelFilenameCharsetCache EelFilenameCharsetCache;
-
-struct _EelFilenameCharsetCache {
-  gboolean is_utf8;
-  gchar *charset;
-  gchar *filename_charset;
-};
-
-static void
-filename_charset_cache_free (gpointer data)
-{
-  EelFilenameCharsetCache *cache = data;
-  g_free (cache->charset);
-  g_free (cache->filename_charset);
-  g_free (cache);
-}
-
-/*
- * eel_get_filename_charset:
- * @charset: return location for the name of the filename encoding 
- *
- * Determines the character set used for filenames by consulting the 
- * environment variables G_FILENAME_ENCODING and G_BROKEN_FILENAMES. 
- *
- * G_FILENAME_ENCODING may be set to a comma-separated list of character 
- * set names. The special token "@locale" is taken to mean the character set 
- * for the current locale. The first character set from the list is taken 
- * as the filename encoding. 
- * If G_FILENAME_ENCODING is not set, but G_BROKEN_FILENAMES is, the
- * character set of the current locale is taken as the filename encoding.
- *
- * The returned @charset belongs to Eel and must not be freed.
- * 
- * Return value: %TRUE if the charset used for filename is UTF-8.
- */
-gboolean
-eel_get_filename_charset (const gchar **filename_charset)
-{
-  static GStaticPrivate cache_private = G_STATIC_PRIVATE_INIT;
-  EelFilenameCharsetCache *cache = g_static_private_get (&cache_private);
-  const gchar *charset;
-  
-  if (!cache)
-    {
-      cache = g_new0 (EelFilenameCharsetCache, 1);
-      g_static_private_set (&cache_private, cache, filename_charset_cache_free);
-    }
-
-  g_get_charset (&charset);
-
-  if (!(cache->charset && strcmp (cache->charset, charset) == 0))
-    {
-      const gchar *new_charset;
-      gchar *p, *q;
-
-      g_free (cache->charset);
-      g_free (cache->filename_charset);
-      cache->charset = g_strdup (charset);
-      
-      p = getenv ("G_FILENAME_ENCODING");
-      if (p != NULL) 
-	{
-	  q = strchr (p, ',');
-	  if (!q) 
-	    q = p + strlen (p);
-
-	  if (strncmp ("@locale", p, q - p) == 0)
-	    {
-	      cache->is_utf8 = g_get_charset (&new_charset);
-	      cache->filename_charset = g_strdup (new_charset);
-	    }
-	  else
-	    {
-	      cache->filename_charset = g_strndup (p, q - p);
-	      cache->is_utf8 = (strcmp (cache->filename_charset, "UTF-8") == 0);
-	    }
-	}
-      else if (getenv ("G_BROKEN_FILENAMES") != NULL)
-	{
-	  cache->is_utf8 = g_get_charset (&new_charset);
-	  cache->filename_charset = g_strdup (new_charset);
-	}
-      else 
-	{
-	  cache->filename_charset = g_strdup ("UTF-8");
-	  cache->is_utf8 = TRUE;
-	}
-    }
-
-  if (filename_charset)
-    *filename_charset = cache->filename_charset;
-
-  return cache->is_utf8;
-}
-
 #if !defined (EEL_OMIT_SELF_CHECK)
 
 static void 
@@ -1183,17 +1117,19 @@ eel_self_check_glib_extensions (void)
 	memset (huge_string, 'a', 10000);
 	huge_string[10000] = '\0';
 
-	setlocale (LC_TIME, "C");
-
 	EEL_CHECK_STRING_RESULT (test_strftime ("", 2000, 1, 1, 0, 0, 0), "");
 	EEL_CHECK_STRING_RESULT (test_strftime (huge_string, 2000, 1, 1, 0, 0, 0), huge_string);
 	EEL_CHECK_STRING_RESULT (test_strftime ("%%", 2000, 1, 1, 1, 0, 0), "%");
 	EEL_CHECK_STRING_RESULT (test_strftime ("%%%%", 2000, 1, 1, 1, 0, 0), "%%");
-	EEL_CHECK_STRING_RESULT (test_strftime ("%m/%d/%y, %I:%M %p", 2000, 1, 1, 1, 0, 0), "01/01/00, 01:00 AM");
-	EEL_CHECK_STRING_RESULT (test_strftime ("%-m/%-d/%y, %-I:%M %p", 2000, 1, 1, 1, 0, 0), "1/1/00, 1:00 AM");
-	EEL_CHECK_STRING_RESULT (test_strftime ("%_m/%_d/%y, %_I:%M %p", 2000, 1, 1, 1, 0, 0), " 1/ 1/00,  1:00 AM");
-
-	setlocale (LC_TIME, "");
+	/* localizers: These strings are part of the strftime
+	 * self-check code and must be changed to match what strtfime
+	 * yields. The first one is "%m/%d/%y, %I:%M %p".
+	 */
+	EEL_CHECK_STRING_RESULT (test_strftime ("%m/%d/%y, %I:%M %p", 2000, 1, 1, 1, 0, 0), _("01/01/00, 01:00 AM"));
+	/* The second one is "%-m/%-d/%-y, %-I:%M %p". */
+	EEL_CHECK_STRING_RESULT (test_strftime ("%-m/%-d/%y, %-I:%M %p", 2000, 1, 1, 1, 0, 0), _("1/1/00, 1:00 AM"));
+	/* The third one is "%_m/%_d/%_y, %_I:%M %p". */
+	EEL_CHECK_STRING_RESULT (test_strftime ("%_m/%_d/%y, %_I:%M %p", 2000, 1, 1, 1, 0, 0), _(" 1/ 1/00,  1:00 AM"));
 
 	g_free (huge_string);
 
